function [X,Y,Z]= monge(alpha,r,th0,xx,yy,ProfilePos,RPfactor)
%computes a Monge surface determined by a planar profile curve alpha
% and a 3D spine curve delta.
%Examples: 
%[X,Y,Z]=monge(@(t)[cos(t);sin(t)],@(t)[cos(t);sin(t);exp(t)],pi, linspace(0,3*pi/2,101),-pi/4:(5/4)*pi/100:pi/4, 51 );
%[X,Y,Z]=monge(@(t)[cos(t);sin(t)],@(t)[cos(t);sin(t);exp(t)],pi, linspace(0,3*pi/2,101),-pi/4:(5/4)*pi/100:pi/4, 1 );

 %Input: 
 %- alpha is either a function handle for the profile curve of form @(t)[x(y); y(t)]
 % or a 2 x m matrix of mesh-data for the profile curve.
 %- r is the spine curve, either a function handle of the 
 %  form @(t)[x(t); y(t); z(t)], or a 3 x m matrix of mesh-data for the curve
 % - theta0 is a rotation of the initial plane containing the profile curve

 %xx =[x1,x2,...,xm] is a row-vector of points, x1<x2<... on which the profile 
 %curve is defined - ignored if alpha is given as a mesh-matrix
 %yy is similarly a row-vector of points on which the spine curve is defined -
 %ignored if r is given as a mesh-matrix
 
 %- ProfilePos: number between xx(1) and xx(end), designates which point on the profile curve 
 %corresponds to the initial point on the spine curve (default is the first point).
 % -if RPfactor >0  (default is 10) then the curve r is reparamterized to have
 % approximately constant speed, by inserting RPfactor points between each parameter point
 % and resampling.  Set to 0 to use the input  parameterization. 
 %(The profile curve alpha is not reparameterized).
 
 %Output:
 %X, Y, and Z are the m by n matrices of the x,y and z coordinate values of
 %the solution surface. Can be plotted with surf(X,Y,Z)
if nargin<7
    RPfactor=10;
end
if nargin<6
    ProfilePos=1;
end
if nargin<3
   th0=0;  
end
%evaluate alpha and translate the curve so that the first point is at the origin
if isa(alpha,'function_handle')
   m=length(xx);
   pos=max(1,min(ProfilePos,m));
   alpha0=(alpha(xx)-alpha(xx(pos)))';
else
     [~,m]=size(alpha);
    pos=max(1,min(ProfilePos,m));
    alpha0=(alpha-alpha(:,pos:pos))';
end

if isa(r,'function_handle')
    n=length(yy);
    if RPfactor %remparameterize at constant speed
         yyy=linspace(yy(1),yy(n),RPfactor*n);
        [~,indices] = constantpar2(r(yyy),n);
        yy=yyy(indices);
    end
    rv=r(yy);
   dy=0.000001;
   drv=(r(yy+dy/2)-r(yy-dy/2))/dy;  
   %2nd derivative at yy(1):
   rtemp=r([yy(1)-dy, yy(1), yy(1)+dy]);
   drtemp=diff(rtemp')'/dy;
   ddr0=diff(drtemp')'/dy;
else
   [~,n]=size(r);
   if RPfactor %remparameterize at constant speed
       r = constantpar(r,RPfactor);
   end
   rv=r;
   if nargin>=5
       ystep=abs(yy(2)-yy(1));
   else
       ystep=1/n;
   end
   drtemp=diff(rv')'/ystep;
   drv=[drtemp, drtemp(:,end:end)];
   ddr0=diff(drtemp(:,1:2)')'/ystep;
end

tang=zeros(3,n);
for j=1:n
    tang(:,j:j)=drv(:,j:j)/norm(drv(:,j:j));
end
tan0=tang(:,1:1);
%calculate the normal to the curve and the corresponding binormal
if norm(ddr0)>0.0000000001
  dpp=ddr0/norm(ddr0);
else  %assumes the curvature is zero 
    %find the smallest component of tan0:
   [~,I]=sort(abs(tan0));
   dpp=[0; 0; 0];
   dpp(I(1))=1;
end
   nor0=dpp-dot(dpp,tan0)*tan0;
   nor0=nor0/norm(nor0);
bin0=cross(tan0,nor0);
norm(bin0);
q0=cos(th0)*nor0+sin(th0)*bin0;

[q1,q2] = MRFrame(rv,tang,q0);

T=q1;
N=q2;

X=zeros(m,n); Y=X; Z=X; 
for j=1:n
X(:,j:j)=T(1:1, j:j)*alpha0(:,1:1)+N(1:1,j:j)*alpha0(:,2:2)+rv(1:1,j:j);
Y(:,j:j)=T(2:2,j:j)*alpha0(:,1:1)+N(2:2,j:j)*alpha0(:,2:2)+rv(2:2,j:j);
Z(:,j:j)=T(3:3,j:j)*alpha0(:,1:1)+N(3:3,j:j)*alpha0(:,2:2)+rv(3:3,j:j);
end
surf(X,Y,Z,'Clipping','off','EdgeAlpha',0,'FaceLighting','gouraud',...
    'DiffuseStrength',0.9,'SpecularExponent',20.0,...
    'SpecularStrength',0.3,'AmbientStrength',0.38,...
    'FaceColor',[0.5 0.7 0.9]); 
daspect([1 1 1]);
hold on
plot3(X(pos:pos,:), Y(pos:pos,:), Z(pos:pos,:), '-r','LineWidth',2);
plot3(X(:,1:1), Y(:,1:1), Z(:,1:1), '-k','LineWidth',2);
hold off
end

function [curve,indices] = constantpar2(curve,n)
x=curve(1:1,:);
y=curve(2:2,:);
z=curve(3:3,:);
N=length(x);
if nargin<2
    n=N;
end
lengths=[0, sqrt(diff(x).^2+diff(y).^2+diff(z).^2)];
L=sum(lengths);
seglen=L/(n-1);
lensums=zeros(N,1);
for j=1:N
    lensums(j)=sum(lengths(1:j));
end
indices=1:n;
indices(1)=1;
indices(n)=N;
for j=2:n-1
   [~, indices(j)]=min(abs(lensums-(j-1)*seglen));
end
curve=[x(indices); y(indices); z(indices)];
end

function [curve,indices] = constantpar(curve,k)
if nargin<2
    k=50;
end
x=curve(1:1,:);
y=curve(2:2,:);
z=curve(3:3,:);
n=length(x);

tq=linspace(1,n,k*n);
x=interp1(x,tq);
y=interp1(y,tq);
z=interp1(z,tq);

lengths=[0, sqrt(diff(x).^2+diff(y).^2+diff(z).^2)];
L=sum(lengths);
seglen=L/(n-1);

lensums=zeros(k*n,1);
for j=1:k*n
    lensums(j)=sum(lengths(1:j));
end
indices=1:n;
indices(1)=1;
indices(n)=k*n;
for j=2:n-1
   [~, indices(j)]=min(abs(lensums-(j-1)*seglen));
end
curve=[x(indices); y(indices); z(indices)];
end




function [q1,q2,kappa1,kappa2] = MRFrame(x,t,q0,dx,d2x)
%
% Input:
% x:   3xn matrix of positions of spine curve
% t:   3xn matrix of unit tangents
% q1:  3x1 unit vector orthogonal to t(:,1)
% dx:  3xn matrix of derivatives of spine curve
% d2x: 3xn matrix of second derivatives of spine curve (only needed
% for curvature)
%
% Output:
% q1: 3xn matrix of unit vectors
% q2: 3xn matrix of unit vectors
% such that
% (t,q1,q2) is a minimal rotating frame
% kappa1, kappa2: 1xn matrix of curvatures
%
n       = numel(t)/3; 
q1      = zeros(3,n);
q1(:,1) = q0;

% First reflexion:
v1 = x(:,2:end) - x(:,1:end-1);
c1 = sum(v1.*v1);
t1 = t(:,1:end-1) - repmat(2*sum((v1.*t(:,1:end-1)))./c1,3,1).*v1;

% Second reflexion:
v2 = t(:,2:end) - t1;
c2 = sum(v2.*v2);

for i = 1:n-1
    q         = q1(:,i) - 2/c1(i)*(v1(:,i)'*q1(:,i))*v1(:,i);
    q1(:,i+1) = q - 2/c2(i)*(v2(:,i)'*q)*v2(:,i);
end

if nargout > 1
    q2 = cross(t,q1);
    if nargout > 2
        dsdt2  = sum(dx.*dx);
        kappa1 = sum(q1.*d2x)./dsdt2;
        kappa2 = sum(q2.*d2x)./dsdt2;
    end
end
end