function plotstrips(X,Y,Z,offset,width,numstrips,color1,color2,style,color3)
%Plots a "wood strip" model of a surface. The strips run in the "vertical"
%direction of the grid.
% See the pdf file "COMPUTING MONGE SURFACES AND WOOD STRIP MODELS"
% at http://geometry.compute.dtu.dk/software  for some background.

%Input:  X,Y and Z are mxn matrices, grid data for a surface.
% offset: a number
% width: a number, usually between 0 and 1
%: numstrips: and integer (should smaller than n, e.g. n/10).
% color1, color2 and color3:  colors, e.g. [0.5, 1, 1] or 'r'
%style:  either 1, 2 or 3

%Example: 
% [X,Y] = meshgrid(1:0.01:10,1:0.05:10);
%Z = sin(X) + cos(Y);
% plotstrips(X,Y,Z,0.15,7,40);
%plotstrips(X,Y,Z,0.15,0.7,40,[1 1 1],[0. 0.9 0.5],1,[1 0 0]);

%Author:  David Brander (dbra@dtu.dk)
if nargin<9
    style=1;
end
if nargin<8
    color2=[0.6 0.6 0.6];
end
if nargin<7
    color1=[1 1 1];
end
if nargin<10
    color3=color1;
end
[~,ypoints]=size(X);
if numstrips==1
    xcurve=floor(ypoints/2);
else
    xstep=floor(ypoints/numstrips);
    xcurve=(0:xstep:(ypoints-xstep))+floor(xstep/2);
end
[~,npointsv]=size(X);
width=width*(floor(npointsv/numstrips));
if isempty(findall(0,'Type','Figure'))
   figure()
end
if style==1
    myplot(X,Y,Z,color2); 
     hold on; daspect([1 1 1]); grid off; axis off
end
f=[X;Y;Z];
[N,~,Tv,B]=normals2(f);
for j=1:numstrips
    strip=makestrip(f(:,xcurve(j):xcurve(j)),B(:,xcurve(j):xcurve(j)),width,Tv(:,xcurve(j):xcurve(j),:));
    if style==1||style==2
      plotf2s(strip,offset,color1,color3,N(:,xcurve(j):xcurve(j)));
    else
       plotf2s2(strip,offset,color1,color2,N(:,xcurve(j):xcurve(j))); 
    end
end
hold off

end

function strip=makestrip(curve,ytangents,width,T)
xpoints=length(curve)/3;
strip=zeros(3*xpoints,2);
Tnorms=sqrt(T(:,:,1).^2+T(:,:,2).^2+T(:,:,3).^2);
ytangents=ytangents.*[Tnorms;Tnorms;Tnorms];
strip(:,1:1)=curve-0.5*width*ytangents;
strip(:,2:2)=curve+0.5*width*ytangents;
end

function  plotf2s(f,offset,color1,color2,N) 
offset=offset/2;
fdims=size(f);
xpoints=fdims(1)/3;
ypoints=fdims(2);
S=singularities(f);
S=(S+0.1)/(1.1);
N=[N(1:xpoints,:).*S; N(xpoints+1:2*xpoints,:).*S; N(2*xpoints+1:3*xpoints,:).*S];
g1=f+offset*N;    
g2=f-offset*N;    
clear f;
myplot(g1(1:xpoints, :), g1(xpoints+1: 2*xpoints, :),  g1(2*xpoints+1: 3*xpoints, :),color1);
 hold on; daspect([1 1 1]); grid off; axis off
myplot(g2(1:xpoints, :), g2(xpoints+1: 2*xpoints, :),  g2(2*xpoints+1: 3*xpoints, :), color2);

   edge1=[g1(:,1:1), g2(:,1:1)];
   edge2=[g1(:,ypoints:ypoints),g2(:,ypoints:ypoints)];
   edge3=[g1(1:1,:);g2(1:1,:); g1(xpoints+1:xpoints+1,:);g2(xpoints+1:xpoints+1,:); g1(2*xpoints+1:2*xpoints+1,:);g2(2*xpoints+1:2*xpoints+1,:)];
   edge4=[g1(xpoints:xpoints,:);g2(xpoints:xpoints,:); g1(2*xpoints:2*xpoints,:);g2(2*xpoints:2*xpoints,:); g1(3*xpoints:3*xpoints,:);g2(3*xpoints:3*xpoints,:)];  
  myplot(edge1(1:xpoints, :), edge1(xpoints+1: 2*xpoints, :),  edge1(2*xpoints+1: 3*xpoints, :),color2);
   myplot(edge2(1:xpoints, :), edge2(xpoints+1: 2*xpoints, :),  edge2(2*xpoints+1: 3*xpoints, :),color2);
   myplot(edge3(1:2, :), edge3(3: 4, :),  edge3(5: 6, :),color2);
   myplot(edge4(1:2, :), edge4(3: 4, :),  edge4(5: 6, :),color2);
end

function myplot(X,Y,Z,color)
surf(X,Y,Z,...
      'EdgeAlpha',0,'FaceLighting','gouraud','FaceColor',color,...
       'DiffuseStrength',0.8,'SpecularExponent',20.0,'SpecularStrength',0.2,'AmbientStrength',0.35);
end
function  plotf2s2(f,offset,color1,color2,N)
fdims=size(f);
xpoints=fdims(1)/3;
ypoints=fdims(2);
S=singularities(f);
S=(S+0.1)/(1.1);
N=[N(1:xpoints,:).*S; N(xpoints+1:2*xpoints,:).*S; N(2*xpoints+1:3*xpoints,:).*S];
g=f+offset*N;    
myplot(f(1:xpoints, :), f(xpoints+1: 2*xpoints, :),  f(2*xpoints+1: 3*xpoints, :),color1);
 hold on; daspect([1 1 1]); grid off; axis off
myplot(g(1:xpoints, :), g(xpoints+1: 2*xpoints, :),  g(2*xpoints+1: 3*xpoints, :), color2);
   edge1=[f(:,1:1), g(:,1:1)];
   edge2=[f(:,ypoints:ypoints),g(:,ypoints:ypoints)];
   edge3=[f(1:1,:);g(1:1,:); f(xpoints+1:xpoints+1,:);g(xpoints+1:xpoints+1,:); f(2*xpoints+1:2*xpoints+1,:);g(2*xpoints+1:2*xpoints+1,:)];
   edge4=[f(xpoints:xpoints,:);g(xpoints:xpoints,:); f(2*xpoints:2*xpoints,:);g(2*xpoints:2*xpoints,:); f(3*xpoints:3*xpoints,:);g(3*xpoints:3*xpoints,:)];  
   myplot(edge1(1:xpoints, :), edge1(xpoints+1: 2*xpoints, :),  edge1(2*xpoints+1: 3*xpoints, :), color2);
   myplot(edge2(1:xpoints, :), edge2(xpoints+1: 2*xpoints, :),  edge2(2*xpoints+1: 3*xpoints, :),color2);
   myplot(edge3(1:2, :), edge3(3: 4, :),  edge3(5: 6, :), color2);
   myplot(edge4(1:2, :), edge4(3: 4, :),  edge4(5: 6, :),color2);
end


function S=singularities(f)
fdims=size(f);  m=fdims(1)/3; n=fdims(2);
fm=circshift(f,-1);
fm(3*m:3*m,:)=f(3*m:3*m,:); fm(2*m:2*m,:)=f(2*m:2*m,:); fm(m:m,:)=f(m:m,:);
fp=circshift(f,1);
fp(1:1,:)=f(1:1,:); fp(m+1:m+1,:)=f(m+1:m+1,:); fp(2*m+1:2*m+1,:)=f(2*m+1:2*m+1,:);
T1=(1/2)*(fp-fm);
T1v=vectors(T1);

fl=(circshift(f',-1))';
fl(:,n:n)=f(:,n:n);
fr=(circshift(f',1))';
fr(:,1:1)=f(:,1:1);
T2=(1/2)*(fr-fl);
T2v=vectors(T2);
cp=mycp(T1v,T2v);
S=mynorm(cp)./(mynorm(T1v).*mynorm(T2v)); 
end


function x=mycp(A,B)
%A and B are m x n x 3 arrays
%calculates the matrix of cross-products
x=A;
x(:,:,1) = A(:,:,2).*B(:,:,3)-A(:,:,3).*B(:,:,2);
x(:,:,2) = -A(:,:,1).*B(:,:,3)+A(:,:,3).*B(:,:,1);
x(:,:,3) = A(:,:,1).*B(:,:,2)-A(:,:,2).*B(:,:,1);
end

function fv=vectors(f)
fdims=size(f);  m=fdims(1)/3; n=fdims(2);
fv=zeros(m,n,3);
fv(:,:,1)=f(1:m,:);
fv(:,:,2)=f(m+1:2*m,:);
fv(:,:,3)=f(2*m+1:3*m,:);

end


function [N,Nv,T2v,B]=normals2(f)
fdims=size(f);  m=fdims(1)/3; n=fdims(2);

fm=circshift(f,-1);
fm(3*m:3*m,:)=f(3*m:3*m,:); fm(2*m:2*m,:)=f(2*m:2*m,:); fm(m:m,:)=f(m:m,:);
fp=circshift(f,1);
fp(1:1,:)=f(1:1,:); fp(m+1:m+1,:)=f(m+1:m+1,:); fp(2*m+1:2*m+1,:)=f(2*m+1:2*m+1,:);
T1=(1/2)*(fp-fm);
T1v=vectors(T1);

fl=(circshift(f',-1))';
fl(:,n:n)=f(:,n:n);
fr=(circshift(f',1))';
fr(:,1:1)=f(:,1:1);
T2=(1/2)*(fr-fl);
T2(:,1:1)=2*T2(:,1:1);
T2(:,n:n)=2*T2(:,n:n);
T2v=vectors(T2);

temp=mycp(T1v,T2v);
Nv=zeros(m,n,3);
norms=mynorm1(temp);
Nv(:,:,1)=temp(:,:,1)./norms;
Nv(:,:,2)=temp(:,:,2)./norms;
Nv(:,:,3)=temp(:,:,3)./norms;

temp2=mycp(Nv,T1v);
Bv=zeros(m,n,3);
norms2=mynorm1(temp2);
Bv(:,:,1)=temp2(:,:,1)./norms2;
Bv(:,:,2)=temp2(:,:,2)./norms2;
Bv(:,:,3)=temp2(:,:,3)./norms2;
B=zeros(3*m,n);
B(1:m,:)=Bv(:,:,1);
B(m+1:2*m,:)=Bv(:,:,2);
B(2*m+1:3*m,:)=Bv(:,:,3);

N=zeros(3*m,n);
N(1:m,:)=Nv(:,:,1);
N(m+1:2*m,:)=Nv(:,:,2);
N(2*m+1:3*m,:)=Nv(:,:,3);
end

function x=mynorm(A)
x=A(:,:,1).*A(:,:,1)+A(:,:,2).*A(:,:,2)+A(:,:,3).*A(:,:,3);
end

function x=mynorm1(A)
x=sqrt(A(:,:,1).*A(:,:,1)+A(:,:,2).*A(:,:,2)+A(:,:,3).*A(:,:,3));
end



